# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V11::Enums::ReachPlanAgeRangeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED        => "UNSPECIFIED",
  UNKNOWN            => "UNKNOWN",
  AGE_RANGE_18_24    => "AGE_RANGE_18_24",
  AGE_RANGE_18_34    => "AGE_RANGE_18_34",
  AGE_RANGE_18_44    => "AGE_RANGE_18_44",
  AGE_RANGE_18_49    => "AGE_RANGE_18_49",
  AGE_RANGE_18_54    => "AGE_RANGE_18_54",
  AGE_RANGE_18_64    => "AGE_RANGE_18_64",
  AGE_RANGE_18_65_UP => "AGE_RANGE_18_65_UP",
  AGE_RANGE_21_34    => "AGE_RANGE_21_34",
  AGE_RANGE_25_34    => "AGE_RANGE_25_34",
  AGE_RANGE_25_44    => "AGE_RANGE_25_44",
  AGE_RANGE_25_49    => "AGE_RANGE_25_49",
  AGE_RANGE_25_54    => "AGE_RANGE_25_54",
  AGE_RANGE_25_64    => "AGE_RANGE_25_64",
  AGE_RANGE_25_65_UP => "AGE_RANGE_25_65_UP",
  AGE_RANGE_35_44    => "AGE_RANGE_35_44",
  AGE_RANGE_35_49    => "AGE_RANGE_35_49",
  AGE_RANGE_35_54    => "AGE_RANGE_35_54",
  AGE_RANGE_35_64    => "AGE_RANGE_35_64",
  AGE_RANGE_35_65_UP => "AGE_RANGE_35_65_UP",
  AGE_RANGE_45_54    => "AGE_RANGE_45_54",
  AGE_RANGE_45_64    => "AGE_RANGE_45_64",
  AGE_RANGE_45_65_UP => "AGE_RANGE_45_65_UP",
  AGE_RANGE_50_65_UP => "AGE_RANGE_50_65_UP",
  AGE_RANGE_55_64    => "AGE_RANGE_55_64",
  AGE_RANGE_55_65_UP => "AGE_RANGE_55_65_UP",
  AGE_RANGE_65_UP    => "AGE_RANGE_65_UP"
];

1;
