# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V10::Enums::PlaceholderTypeEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED         => "UNSPECIFIED",
  UNKNOWN             => "UNKNOWN",
  SITELINK            => "SITELINK",
  CALL                => "CALL",
  APP                 => "APP",
  LOCATION            => "LOCATION",
  AFFILIATE_LOCATION  => "AFFILIATE_LOCATION",
  CALLOUT             => "CALLOUT",
  STRUCTURED_SNIPPET  => "STRUCTURED_SNIPPET",
  MESSAGE             => "MESSAGE",
  PRICE               => "PRICE",
  PROMOTION           => "PROMOTION",
  AD_CUSTOMIZER       => "AD_CUSTOMIZER",
  DYNAMIC_EDUCATION   => "DYNAMIC_EDUCATION",
  DYNAMIC_FLIGHT      => "DYNAMIC_FLIGHT",
  DYNAMIC_CUSTOM      => "DYNAMIC_CUSTOM",
  DYNAMIC_HOTEL       => "DYNAMIC_HOTEL",
  DYNAMIC_REAL_ESTATE => "DYNAMIC_REAL_ESTATE",
  DYNAMIC_TRAVEL      => "DYNAMIC_TRAVEL",
  DYNAMIC_LOCAL       => "DYNAMIC_LOCAL",
  DYNAMIC_JOB         => "DYNAMIC_JOB",
  IMAGE               => "IMAGE"
];

1;
