# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V10::Enums::PromotionExtensionOccasionEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED      => "UNSPECIFIED",
  UNKNOWN          => "UNKNOWN",
  NEW_YEARS        => "NEW_YEARS",
  CHINESE_NEW_YEAR => "CHINESE_NEW_YEAR",
  VALENTINES_DAY   => "VALENTINES_DAY",
  EASTER           => "EASTER",
  MOTHERS_DAY      => "MOTHERS_DAY",
  FATHERS_DAY      => "FATHERS_DAY",
  LABOR_DAY        => "LABOR_DAY",
  BACK_TO_SCHOOL   => "BACK_TO_SCHOOL",
  HALLOWEEN        => "HALLOWEEN",
  BLACK_FRIDAY     => "BLACK_FRIDAY",
  CYBER_MONDAY     => "CYBER_MONDAY",
  CHRISTMAS        => "CHRISTMAS",
  BOXING_DAY       => "BOXING_DAY",
  INDEPENDENCE_DAY => "INDEPENDENCE_DAY",
  NATIONAL_DAY     => "NATIONAL_DAY",
  END_OF_SEASON    => "END_OF_SEASON",
  WINTER_SALE      => "WINTER_SALE",
  SUMMER_SALE      => "SUMMER_SALE",
  FALL_SALE        => "FALL_SALE",
  SPRING_SALE      => "SPRING_SALE",
  RAMADAN          => "RAMADAN",
  EID_AL_FITR      => "EID_AL_FITR",
  EID_AL_ADHA      => "EID_AL_ADHA",
  SINGLES_DAY      => "SINGLES_DAY",
  WOMENS_DAY       => "WOMENS_DAY",
  HOLI             => "HOLI",
  PARENTS_DAY      => "PARENTS_DAY",
  ST_NICHOLAS_DAY  => "ST_NICHOLAS_DAY",
  CARNIVAL         => "CARNIVAL",
  EPIPHANY         => "EPIPHANY",
  ROSH_HASHANAH    => "ROSH_HASHANAH",
  PASSOVER         => "PASSOVER",
  HANUKKAH         => "HANUKKAH",
  DIWALI           => "DIWALI",
  NAVRATRI         => "NAVRATRI",
  SONGKRAN         => "SONGKRAN",
  YEAR_END_GIFT    => "YEAR_END_GIFT"
];

1;
