# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V10::Enums::DistanceBucketEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED     => "UNSPECIFIED",
  UNKNOWN         => "UNKNOWN",
  WITHIN_700M     => "WITHIN_700M",
  WITHIN_1KM      => "WITHIN_1KM",
  WITHIN_5KM      => "WITHIN_5KM",
  WITHIN_10KM     => "WITHIN_10KM",
  WITHIN_15KM     => "WITHIN_15KM",
  WITHIN_20KM     => "WITHIN_20KM",
  WITHIN_25KM     => "WITHIN_25KM",
  WITHIN_30KM     => "WITHIN_30KM",
  WITHIN_35KM     => "WITHIN_35KM",
  WITHIN_40KM     => "WITHIN_40KM",
  WITHIN_45KM     => "WITHIN_45KM",
  WITHIN_50KM     => "WITHIN_50KM",
  WITHIN_55KM     => "WITHIN_55KM",
  WITHIN_60KM     => "WITHIN_60KM",
  WITHIN_65KM     => "WITHIN_65KM",
  BEYOND_65KM     => "BEYOND_65KM",
  WITHIN_0_7MILES => "WITHIN_0_7MILES",
  WITHIN_1MILE    => "WITHIN_1MILE",
  WITHIN_5MILES   => "WITHIN_5MILES",
  WITHIN_10MILES  => "WITHIN_10MILES",
  WITHIN_15MILES  => "WITHIN_15MILES",
  WITHIN_20MILES  => "WITHIN_20MILES",
  WITHIN_25MILES  => "WITHIN_25MILES",
  WITHIN_30MILES  => "WITHIN_30MILES",
  WITHIN_35MILES  => "WITHIN_35MILES",
  WITHIN_40MILES  => "WITHIN_40MILES",
  BEYOND_40MILES  => "BEYOND_40MILES"
];

1;
