# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V10::Enums::EducationPlaceholderFieldEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                     => "UNSPECIFIED",
  UNKNOWN                         => "UNKNOWN",
  PROGRAM_ID                      => "PROGRAM_ID",
  LOCATION_ID                     => "LOCATION_ID",
  PROGRAM_NAME                    => "PROGRAM_NAME",
  AREA_OF_STUDY                   => "AREA_OF_STUDY",
  PROGRAM_DESCRIPTION             => "PROGRAM_DESCRIPTION",
  SCHOOL_NAME                     => "SCHOOL_NAME",
  ADDRESS                         => "ADDRESS",
  THUMBNAIL_IMAGE_URL             => "THUMBNAIL_IMAGE_URL",
  ALTERNATIVE_THUMBNAIL_IMAGE_URL => "ALTERNATIVE_THUMBNAIL_IMAGE_URL",
  FINAL_URLS                      => "FINAL_URLS",
  FINAL_MOBILE_URLS               => "FINAL_MOBILE_URLS",
  TRACKING_URL                    => "TRACKING_URL",
  CONTEXTUAL_KEYWORDS             => "CONTEXTUAL_KEYWORDS",
  ANDROID_APP_LINK                => "ANDROID_APP_LINK",
  SIMILAR_PROGRAM_IDS             => "SIMILAR_PROGRAM_IDS",
  IOS_APP_LINK                    => "IOS_APP_LINK",
  IOS_APP_STORE_ID                => "IOS_APP_STORE_ID"
];

1;
