# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V10::Enums::VanityPharmaTextEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                           => "UNSPECIFIED",
  UNKNOWN                               => "UNKNOWN",
  PRESCRIPTION_TREATMENT_WEBSITE_EN     => "PRESCRIPTION_TREATMENT_WEBSITE_EN",
  PRESCRIPTION_TREATMENT_WEBSITE_ES     => "PRESCRIPTION_TREATMENT_WEBSITE_ES",
  PRESCRIPTION_DEVICE_WEBSITE_EN        => "PRESCRIPTION_DEVICE_WEBSITE_EN",
  PRESCRIPTION_DEVICE_WEBSITE_ES        => "PRESCRIPTION_DEVICE_WEBSITE_ES",
  MEDICAL_DEVICE_WEBSITE_EN             => "MEDICAL_DEVICE_WEBSITE_EN",
  MEDICAL_DEVICE_WEBSITE_ES             => "MEDICAL_DEVICE_WEBSITE_ES",
  PREVENTATIVE_TREATMENT_WEBSITE_EN     => "PREVENTATIVE_TREATMENT_WEBSITE_EN",
  PREVENTATIVE_TREATMENT_WEBSITE_ES     => "PREVENTATIVE_TREATMENT_WEBSITE_ES",
  PRESCRIPTION_CONTRACEPTION_WEBSITE_EN =>
    "PRESCRIPTION_CONTRACEPTION_WEBSITE_EN",
  PRESCRIPTION_CONTRACEPTION_WEBSITE_ES =>
    "PRESCRIPTION_CONTRACEPTION_WEBSITE_ES",
  PRESCRIPTION_VACCINE_WEBSITE_EN => "PRESCRIPTION_VACCINE_WEBSITE_EN",
  PRESCRIPTION_VACCINE_WEBSITE_ES => "PRESCRIPTION_VACCINE_WEBSITE_ES"
];

1;
